use strict;
use Test::Simple tests => 18;

use DE_EPAGES::Core::API::File qw ( CopyFile );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( ConnectTestSite DisconnectSite );
use DE_EPAGES::ImageGallery::API::Cartridge;
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    InsertObject
    LoadObjectByPath
    LoadRootObject
);

my $ROOTPATH = '/Test';

## no critic qw(ProhibitMagicNumbers)
sub Main {
    ConnectTestSite();

    # create test classes
    ok( Init(), 'initialization' );

    my $TestShop = LoadObjectByPath( $ROOTPATH );

    my $TestGallery = InsertObject('ImageGallery', {
        Parent => $TestShop,
        Alias => 'TestGallery',
        ImageDirectory => 'examples2',
        ThumbnailSize => 100,
        MediumSize => 350
    });
    ok(defined $TestGallery, 'ImageGallery inserted');
    ok(LoadObjectByPath( $ROOTPATH )->existsChild('TestGallery'), 'ImageGallery exists');


    $TestGallery = LoadObjectByPath( $ROOTPATH )->child('TestGallery');
    ok( $TestGallery->get('ImageDirectory') eq 'examples2' &&
        $TestGallery->get('ThumbnailSize') == 100 &&
        $TestGallery->get('MediumSize') == 350,
        'ImageGallery has the right values');

    $TestGallery->set({'ImageDirectory' => 'examples'});
    ok(defined $TestGallery, 'ImageGallery updated');

    $TestGallery = LoadObjectByPath($ROOTPATH)->child('TestGallery');
    ok( $TestGallery->get('ImageDirectory') eq 'examples',
        'ImageGallery has the right values');


    my $TestImage = InsertObject('GalleryImage', {
        Parent => $TestGallery,
        Alias => 'TestImage',
        ImageGallery => $TestGallery
    });
    ok(defined $TestImage, 'GalleryImage inserted');

    $TestImage = LoadObjectByPath($ROOTPATH)->folder('TestGallery')->child('TestImage');
    $TestImage->set({
        'Tags' => ['cool','lustig','Bild'],
        'Date' => '2007-05-23 12:34:56'
    });
    ok(defined $TestImage, 'GalleryImage updated');

    $TestImage = LoadObjectByPath($ROOTPATH)->folder('TestGallery')->child('TestImage');
    my $aTagNames = $TestImage->get('TagNames');
    my $TestString = join ',', @$aTagNames ;

    ok( $TestString eq 'Bild,cool,lustig', 'GalleryImage has the right values');

    $TestImage->set({'MediaGalleryFile' => 'examples/image01.jpg'});
    ok(defined $TestImage, 'GalleryImage has got a MediaGalleryAttribute');

    $TestImage = LoadObjectByPath($ROOTPATH)->folder('TestGallery')->child('TestImage');
    ok( $TestImage->get('MediaGalleryFile') eq 'examples/image01.jpg',
        'MediaGalleryAttribute has the right value');

    $TestGallery->addImagesFromDirectory('examples');
    ok( $TestGallery, 'addImagesFromDirectory finished' );

    # sort alphabetically by MediaGalleryFile (name)
    my @GalleryImages = sort { $a->get('MediaGalleryFile') cmp $b->get('MediaGalleryFile') } @{$TestGallery->get('Images')};

    ok ($GalleryImages[0]->get('MediaGalleryFile') eq 'examples/image01.jpg' &&
        $GalleryImages[1]->get('MediaGalleryFile') eq 'examples/image02.jpg' &&
        $GalleryImages[2]->get('MediaGalleryFile') eq 'examples/image03.jpg',
        'GalleryImages were added correctly');

    my $MediaGallery = $TestShop->get('MediaGallery');

    # delete a gallery image (image02.jpg) including its previews
    my $OriginalFileName = $GalleryImages[1]->get('OriginalImage');
    $GalleryImages[1]->delete;

    my $MappedFileName = $MediaGallery->attribute2galleryFileName($OriginalFileName);
    ok( !$MediaGallery->existsFile($MappedFileName), "original file ('$MappedFileName') deleted" );

    foreach my $PreviewType (@{$TestGallery->previewTypes}) {
        my $PreviewImage = $MediaGallery->attribute2galleryFileName($MediaGallery->previewPath($OriginalFileName, $PreviewType));
        ok( !$MediaGallery->existsFile($PreviewImage), "preview image ('$PreviewImage') filename deleted" );
    }

    # finally, delete entire test gallery
    my $ImageDirectory = $TestGallery->get('ImageDirectory');
    $TestGallery->delete;
    ok ( !$MediaGallery->existsDirectory($ImageDirectory), 'testgallery deleted' );

    ok( Cleanup(), 'cleanup' );

    DisconnectSite();
}

sub Cleanup {
    if( ExistsObjectByPath( "$ROOTPATH/TestGallery/TestImage") ) { ##no critic (ProhibitPathConcatenation)
        LoadObjectByPath(  $ROOTPATH )->child('TestGallery')->child('TestImage')->delete;
    }
    if( ExistsObjectByPath( "$ROOTPATH/TestGallery" ) ) {##no critic (ProhibitPathConcatenation)
        LoadObjectByPath( $ROOTPATH )->child('TestGallery')->delete;
    }
    if( ExistsObjectByPath( $ROOTPATH ) ) {
        LoadObjectByPath( $ROOTPATH )->delete;
    }
    return 1;
}

sub Init {
    Cleanup();
    my $Shop = InsertObject( 'Shop', { Alias => 'Test', Parent => LoadRootObject()} );
    $Shop->featureMaxValue('GallerySize', 10000000);
    $Shop->featureMaxValue('Pages', 10);
    $Shop->featureMaxValue('ImageGallery', 1);
    # copy files to MediaGallery of Test object
    my $mg = $Shop->child('MediaGallery');
    my $Cartridge = (DE_EPAGES::ImageGallery::API::Cartridge->new);
    my $SourcePath = $Cartridge->xmlDirectory() . '/../../t/Data/';
    $mg->createDirectory('examples');
    CopyFile($SourcePath.'image01.jpg', $mg->get('PublicPath').'/examples/image01.jpg');
    CopyFile($SourcePath.'image02.jpg', $mg->get('PublicPath').'/examples/image02.jpg');
    CopyFile($SourcePath.'image03.jpg', $mg->get('PublicPath').'/examples/image03.jpg');
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
