#========================================================================================
# §package      Test
# §state        private
# §description  Test if feature check in Hooks/Category works correctly.
#
#               Please note full functional test would requiere to mock Servlet object
#               completely, which is not feasible. Instead we do a partial mocking, and
#               catch errors produced by unmocked Servlet methods. From existence
#               of an error and source of an error, we can deduce if code logic is working 
#               as intended. 
#========================================================================================
package Test;
use strict;

use Test::More tests => 14;
use DE_EPAGES::RemoteSearch::API::RemoteSearchImpl qw ( UseSearchInCategoryContext );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::RemoteSearch::Hooks::Category;

sub TestUseSearchInCategoryContext{
    my $Shop = MockShop->new();
    $Shop->set_isa('DE_EPAGES::Shop::API::Object::Shop');
    $Shop->set('DetailedSearch', 1);
    is(1 , UseSearchInCategoryContext($Shop), 'Merchant can see MBO > Setting > Search');
    $Shop->set('DetailedSearch', 0);
    is(0 , UseSearchInCategoryContext($Shop), 'Merchant can\'t see MBO > Setting > Search');
}

sub TestOnCategoryProductsPager {
    # avoid TestObject call and Database access
    local *DE_EPAGES::RemoteSearch::Hooks::Category::GetPageSizeForCategory = sub ($){shift; return 20};

    # shop with feature DetailSearch (no MBO > Setting > Search menu)
    my $hParams->{Category} = MockCategory->new();
    $hParams->{PagerVars}->{Servlet} = MockServlet->new();
    $hParams->{Category}->initShopWithDetailedSearch();
    $hParams->{Category}->set('CategoryPathDepth',1);
    eval {DE_EPAGES::RemoteSearch::Hooks::Category::OnCategoryProductsPager( $hParams );};
    ok( ! ExistsError(), 'don\'t trigger search - CategoryPathDepth < Shop.FacetedSearchStartDepth');

    # we expect error in FacetedSearch.pm on servlet method call
    $hParams->{Category}->set('CategoryPathDepth',2);
    eval {DE_EPAGES::RemoteSearch::Hooks::Category::OnCategoryProductsPager( $hParams );};
    ok( ExistsError && GetError->message() =~ m/FacetedSearch/x,
        'trigger search - start CategoryPathDepth == Shop.FacetedSearchStartDepth') or diag(GetError);

    # we expect error in FacetedSearch.pm on servlet method call
    $hParams->{Category}->set('CategoryPathDepth',3);
    eval {DE_EPAGES::RemoteSearch::Hooks::Category::OnCategoryProductsPager( $hParams );};
    ok( ExistsError && GetError->message() =~ m/FacetedSearch/x,
        'trigger search - CategoryPathDepth > Shop.FacetedSearchStartDepth') or diag(GetError);
}

sub TestOnCategoryProductsPager2 {
    # shop without feature DetailSearch (no MBO > Setting > Search menu)
    my $hParams->{Category} = MockCategory->new();
    $hParams->{PagerVars}->{Servlet} = MockServlet->new();
    $hParams->{Category}->initShopWithOutDetailedSearch();
    $hParams->{Category}->set('CategoryPathDepth',1);
    eval {DE_EPAGES::RemoteSearch::Hooks::Category::OnCategoryProductsPager( $hParams );};
    ok( ! ExistsError(), 'no feature -> no search - CategoryPathDepth < Shop.FacetedSearchStartDepth');

    $hParams->{Category}->set('CategoryPathDepth',2);
    eval {DE_EPAGES::RemoteSearch::Hooks::Category::OnCategoryProductsPager( $hParams );};

    ok( ! ExistsError(), 'no feature -> no search - start CategoryPathDepth == Shop.FacetedSearchStartDepth');

    $hParams->{Category}->set('CategoryPathDepth',3);
    eval {DE_EPAGES::RemoteSearch::Hooks::Category::OnCategoryProductsPager( $hParams );};
    ok( ! ExistsError(), 'no feature -> no search - CategoryPathDepth > Shop.FacetedSearchStartDepth');

}

sub TestOnCategoryProductsPager3 {
    # shop with feature DetailSearch (MBO > Setting > Search menu is there) but Merchant has deactivated search in category context
    my $hParams->{Category} = MockCategory->new();
    $hParams->{PagerVars}->{Servlet} = MockServlet->new();
    $hParams->{Category}->initShopWithDetailedSearchButStartDepthZero();
    $hParams->{Category}->set('CategoryPathDepth',1);
    eval {DE_EPAGES::RemoteSearch::Hooks::Category::OnCategoryProductsPager( $hParams );};
    ok( ! ExistsError(), 'Shop.FacetedSearchStartDepth = 0 -> no search - CategoryPathDepth < Shop.FacetedSearchStartDepth');

    $hParams->{Category}->set('CategoryPathDepth',2);
    eval {DE_EPAGES::RemoteSearch::Hooks::Category::OnCategoryProductsPager( $hParams );};

    ok( ! ExistsError(), 'Shop.FacetedSearchStartDepth = 0 -> no search - start CategoryPathDepth == Shop.FacetedSearchStartDepth');

    $hParams->{Category}->set('CategoryPathDepth',3);
    eval {DE_EPAGES::RemoteSearch::Hooks::Category::OnCategoryProductsPager( $hParams );};
    ok( ! ExistsError(), 'Shop.FacetedSearchStartDepth = 0 -> no search - CategoryPathDepth > Shop.FacetedSearchStartDepth');
}

sub TestOnCategoryProductsPager4 {
    # shop without feature DetailSearch (no menu MBO > Setting > Search) but Merchant has deactivated search in category context
    my $hParams->{Category} = MockCategory->new();
    $hParams->{PagerVars}->{Servlet} = MockServlet->new();
    $hParams->{Category}->initShopWithOutDetailedSearchAndStartDepthZero();
    $hParams->{Category}->set('CategoryPathDepth',1);
    eval {DE_EPAGES::RemoteSearch::Hooks::Category::OnCategoryProductsPager( $hParams );};
    ok( ! ExistsError(),
        'Shop.FacetedSearchStartDepth = 0 and no DetailSearch -> no search - CategoryPathDepth < Shop.FacetedSearchStartDepth');

    $hParams->{Category}->set('CategoryPathDepth',2);
    eval {DE_EPAGES::RemoteSearch::Hooks::Category::OnCategoryProductsPager( $hParams );};

    ok( ! ExistsError(),
        'Shop.FacetedSearchStartDepth = 0 and no DetailSearch -> no search - start CategoryPathDepth == Shop.FacetedSearchStartDepth');

    $hParams->{Category}->set('CategoryPathDepth',3);
    eval {DE_EPAGES::RemoteSearch::Hooks::Category::OnCategoryProductsPager( $hParams );};
    ok( ! ExistsError(),
        'Shop.FacetedSearchStartDepth = 0 and no DetailSearch -> no search - CategoryPathDepth > Shop.FacetedSearchStartDepth');
}

RunScript(
    Sub => sub {
        TestUseSearchInCategoryContext();
        TestOnCategoryProductsPager();
        TestOnCategoryProductsPager2();
        TestOnCategoryProductsPager3();
        TestOnCategoryProductsPager4();
    }
);

package MockServlet; ## no critic (ProhibitMultiplePackages)
use base qw ( DE_EPAGES::Core::API::Object::BaseObject );

use strict;

sub vars {}

sub params { shift; return { 'ViewAction' => 'View'}->{shift} }

package MockShop; ## no critic (ProhibitMultiplePackages)
use base qw ( DE_EPAGES::Core::API::Object::BaseObject Test::MockObject);
use strict;

sub featureMaxValue {
    my $self = shift;
    my ($Feature) = @_;

    return $self->get($Feature);
}

package MockCategory; ## no critic (ProhibitMultiplePackages)
use base qw ( DE_EPAGES::Core::API::Object::BaseObject);
use strict;

sub getMockShop {
    my $self = shift;
    my $Shop = MockShop->new();
    $Shop->set_isa('DE_EPAGES::Shop::API::Object::Shop');
    return $Shop;
}

sub initShopWithDetailedSearch {
    my $self = shift;
    my $Shop = $self->getMockShop();
    $Shop->set('FacetedSearchStartDepth', 2);
    $Shop->set('DetailedSearch', 1);
    $Shop->set('FacetedSearchShowFacets', 1);
    $self->set('Shop', $Shop);
}

sub initShopWithDetailedSearchButStartDepthZero {
    my $self = shift;
    my $Shop = $self->getMockShop();
    $Shop->set('FacetedSearchStartDepth', 0);
    $Shop->set('DetailedSearch', 1);
    $Shop->set('FacetedSearchShowFacets', 1);
    $self->set('Shop', $Shop);
}

sub initShopWithOutDetailedSearch {
    my $self = shift;
    my $Shop = $self->getMockShop();
    $Shop->set('FacetedSearchStartDepth', 2);
    $Shop->set('DetailedSearch', 0);
    $Shop->set('FacetedSearchShowFacets', 1);
    $self->set('Shop', $Shop);
}

sub initShopWithOutDetailedSearchAndStartDepthZero {
    my $self = shift;
    my $Shop = $self->getMockShop();
    $Shop->set('FacetedSearchStartDepth', 0);
    $Shop->set('DetailedSearch', 0);
    $Shop->set('FacetedSearchShowFacets', 1);
    $self->set('Shop', $Shop);
}

sub getSite {
    my $self = shift;

    return $self->get('Shop');
}

1;