use strict;
use threads;

use Getopt::Long;
use JSON;
use Sort::Versions qw(versioncmp);
use Test::More;
use Test::Deep;

use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::File qw( WriteFile DeleteFile GetFileContent );
use DE_EPAGES::Core::API::PerlTools qw( UsePackage );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw( ExistsObjectByPath LoadObjectByPath );
use DE_EPAGES::Presentation::API::ContentAnalyzer qw( FindLinkedPages );
use DE_EPAGES::WebInterface::API::Crawler::Crawler;
use DE_EPAGES::WebInterface::API::Crawler::Requester;

BEGIN {
    UsePackage('POSIX') if $^O eq 'linux';
}

if ($^O eq 'linux') {
    plan skip_all => 'Test runs only on 64-bit redhead 6' unless ( versioncmp((POSIX::uname)[2], '2.6.27') >= 0 && (POSIX::uname)[4] eq 'x86_64')
}

if ($ENV{'SKIP_NON_WORKING'}) {
    plan skip_all => 'Test does not work on patched installations.'
}

sub Main {
    SKIP: {
        skip 'Not running on site' unless GetTestSite() eq 'Site';
        my $Provider = LoadObjectByPath('/Providers/Distributor');
        my $ShopTypes =  $Provider->child('ShopTypes');

        my $Store = LoadObjectByPath('/Stores/Store');

        foreach my $ShopType(@{$ShopTypes->children}) {

            my $ShopAlias = 'MicroDataTestShop_' . $ShopType->alias;

            LoadObjectByPath('/Providers/Distributor/ShopRefs/' .  $ShopAlias)->delete if(ExistsObjectByPath('/Providers/Distributor/ShopRefs/' .  $ShopAlias));

            $Provider->insertShop({
                ShopType => $ShopType,
                Store => $Store,
                Alias => $ShopAlias,
                ShopAlias => $ShopAlias,
                WebServerScriptNamePart => $ShopAlias
            });

            my $Shop = LoadObjectByPath('/Providers/Distributor/ShopRefs/' .  $ShopAlias);

            my $StoreFrontURL = $Shop->get('WebUrl');

            my $Requester = DE_EPAGES::WebInterface::API::Crawler::Requester->new();

            # Find all reachable storefront pages -> change to imported demoshop
            my $SFCrawler = DE_EPAGES::WebInterface::API::Crawler::Crawler->new( $Requester, $StoreFrontURL, \&FindLinkedPages );
            $SFCrawler->start();

            my $ExpectedMicroDataContent = GetFileContent('expected_microdata_' . $ShopAlias . '.json');
            my $ExpectedMicroData = from_json($$ExpectedMicroDataContent);

            my $MicroData = {};

            # Find all forms and input fields
            foreach my $Page ( keys %{$SFCrawler->{'pages'}} ) {
                ok($SFCrawler->{'pages'}->{$Page}->{'statuscode'} < 400, "Status code for $Page should be < 400");

                # write content to tmp file
                my $Content = $SFCrawler->{'pages'}->{$Page}->{'content'};
                $Content =~ s/<base.*?>//;
                WriteFile('pagetotest.html', \$Content, {'utf8'=>1});

                my $Command = "nodejs.sh microdataextract.js pagetotest.html";
                my $Json = `$Command`;
                my $PageName = ($Page =~ /^https?:\/\/.*?(\/.*)/) ? $1 : $Page;
                $MicroData->{$PageName} = from_json($Json);
                cmp_deeply($MicroData->{$PageName}, $ExpectedMicroData->{$PageName}, 'MicroData should be equal to expectation');
            }

            # delete tmp file
            DeleteFile('pagetotest.html');

            # use the line below to write the microdata to file and set the new expectation
            # WriteFile('expected_microdata_' . $ShopAlias . '.json',  \to_json($MicroData, {pretty => 1}) );

            $Shop->delete;
        }
    }

    done_testing();
}

RunScript(Sub => sub {RunOnStore('Store' => GetTestSite(),'Sub' => \&Main , Debug=>1);});

1;
